// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vbsocket.cpp
// C++ Compiler Used: MSVC40, HP CPP 10.24, egcs-2.90.29 
// Produced By: Doug Gaer   
// File Creation Date: 09/20/1999
// Date Last Modified: 05/26/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBS C++ classes are copyright (c) 2000, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The vbSocket class is used to create TCP/IP sockets. The vbSocket
class supports stream sockets and datagram sockets It includes
several low-level functions needed by derived classes to establish
communication end-points and transfer data.
*/
// ----------------------------------------------------------- // 
#include<string.h>
#include <fcntl.h>
#include "vbsocket.h"

#if defined(__UNIX__) && !defined(__BSD_UNIX__)  
#include <sys/utsname.h> // Used for host name information
#endif // __UNIX__

// NOTE: This array must contain the same number of exceptions as the
// vbsSocketError enumeration. 
const char *vbsSocketExceptionMessages[14] = {
  "Socket exception: No exception reported",         // NO_ERROR
  "Socket exception: Invalid exception code",        // INVALID_CODE 
  "Socket exception: Error accepting remote socket", // ACCEPT_ERROR
  "Socket exception: Could not bind socket",         // BIND_ERROR
  "Socket exception: Could not connect socket",      // CONNECT_ERROR
  "Socket exception: Could not resolve hostname",    // HOSTNAME_ERROR
  "Socket exception: Initialization error",          // INIT_ERROR
  "Socket exception: Listen error",                  // LISTEN_ERROR
  "Socket exception: Unknown protocol requested"     // PROTOCOL_ERROR
  "Socket exception: Receive error",                 // RECEIVE_ERROR
  "Socket exception: Unknown service requested",     // SERVICE_ERROR
  "Socket exception: Transmit error",                // TRANSMIT_ERROR
  "Socket exception: Could not inititalize Winsock", // WINSOCKINIT_ERROR
  "Socket exception: Winsock version not supported"  // WINSOCKVERSION_ERROR
};

vbSocket::vbSocket()
{
  address_family = AF_INET;      // Default address family
  socket_type = SOCK_STREAM;     // Default socket type
  protocol_family = 0;           // Default protocol family
  port_number = vbsDEFAULT_PORT; // Default port number
  vbsocket = -1;
  remote_socket = -1;
  bytes_read = bytes_moved = 0;
  is_connected = 0;
  is_bound = 0;
  socket_error = vbSOCKET_NO_ERROR;
}

vbSocket::~vbSocket()
{
  Close();
}

vbsSocketType vbSocket::Socket()
// Create a socket. Returns the a valid socket descriptor or
// -1 if the socket cannot be initializied. 
{
  vbsocket = socket(address_family, socket_type, protocol_family);
  if(vbsocket < 0) socket_error = vbSOCKET_INIT_ERROR;
  return vbsocket;
}

vbsSocketType vbSocket::InitSocket()
// Create a and initialize a socket. This version was added to
// include the WINSOCK initialization functions as part of the
// socket initialization. Returns the a valid socket descriptor
// or -1 if the socket cannot be initializied. 
{
#if defined (__WIN32__)
  WORD ver = MAKEWORD(1,1);
  if(InitWinSock(ver) != 0) {
    socket_error = vbSOCKET_WINSOCKINIT_ERROR;
    return -1; // Could not inititalize Winsock
  }
  else {
    if(!CheckWinSockVersion()) {
      socket_error = vbSOCKET_WINSOCKVERSION_ERROR;
      WSACleanup();
      return -1; // Winsock version not supported
    }
  }
#endif
  vbsocket = socket(address_family, socket_type, protocol_family);
  if(vbsocket < 0) socket_error = vbSOCKET_INIT_ERROR;
  return vbsocket;
}

int vbSocket::Bind()
// Bind the socket to a name so that other processes can
// reference it and allow this socket to receive messages.
// Returns -1 if an error occurs.
{
  int rv = bind(vbsocket, (struct sockaddr *)&sin, sizeof(sin));
  if(rv >= 0) {
    is_bound = 1;
  } 
  else {
    socket_error = vbSOCKET_BIND_ERROR;
    is_bound = 0;
  }
  return rv;
}

int vbSocket::Connect()
// Connect the socket to a client or server. On the client side
// a connect call is used to initiate a connection.
// Returns -1 if an error occurs.
{
  int rv = connect(vbsocket, (struct sockaddr *)&sin, sizeof(sin));
  if(rv >= 0) {
    is_connected = 1; 
  }
  else {
    socket_error = vbSOCKET_CONNECT_ERROR;
    is_connected = 0;
  }
  return rv;
}

int vbSocket::Recv(void *buf, int bytes, int flags)
// Receive a block of data from the bound socket and do not return
// until all the bytes have been read. Returns the total number of 
// bytes received or -1 if an error occurs.
{
  bytes_read = 0;           // Reset the byte counter
  int num_read = 0;         // Actual number of bytes read
  int num_req = (int)bytes; // Number of bytes requested 
  char *p = (char *)buf;    // Pointer to the buffer

  while(bytes_read < bytes) { // Loop unitl the buffer is full
    if((num_read = recv(vbsocket, p, num_req-bytes_read, flags)) > 0) {
      bytes_read += num_read;   // Increment the byte counter
      p += num_read;            // Move the buffer pointer for the next read
    }
    if(num_read < 0) {
      socket_error = vbSOCKET_RECEIVE_ERROR;
      return -1; // An error occurred during the read
    }
  }
  return bytes_read;
}

int vbSocket::Send(const void *buf, int bytes, int flags)
// Send a block of data to the bound socket and do not return
// until all the bytes have been written. Returns the total number 
// of bytes sent or -1 if an error occurs.
{
  bytes_moved = 0;           // Reset the byte counter
  int num_moved = 0;         // Actual number of bytes written
  int num_req = (int)bytes;  // Number of bytes requested 
  char *p = (char *)buf;     // Pointer to the buffer

  while(bytes_moved < bytes) { // Loop unitl the buffer is empty
    if((num_moved = send(vbsocket, p, num_req-bytes_moved, flags)) > 0) {
      bytes_moved += num_moved;  // Increment the byte counter
      p += num_moved;            // Move the buffer pointer for the next read
    }
    if(num_moved < 0) {
      socket_error = vbSOCKET_TRANSMIT_ERROR;
      return -1; // An error occurred during the read
    }
  }
  
  return bytes_moved;
}

int vbSocket::Recv(vbsSocketType s, void *buf, int bytes, int flags)
// Receive a block of data from a specifed socket and do not return
// until all the bytes have been read. Returns the total number of
// bytes received or -1 if an error occurs.
{
  bytes_read = 0;           // Reset the byte counter
  int num_read = 0;         // Actual number of bytes read
  int num_req = (int)bytes; // Number of bytes requested 
  char *p = (char *)buf;    // Pointer to the buffer

  while(bytes_read < bytes) { // Loop unitl the buffer is full
    if((num_read = recv(s, p, num_req-bytes_read, flags)) > 0) {
      bytes_read += num_read;   // Increment the byte counter
      p += num_read;            // Move the buffer pointer for the next read
    }
    if(num_read < 0) {
      socket_error = vbSOCKET_RECEIVE_ERROR;
      return -1; // An error occurred during the read
    }
  }
  
  return bytes_read;
}

int vbSocket::Send(vbsSocketType s, const void *buf, int bytes, int flags)
// Send a block of data to a spcifed socket socket and do not return
// unitl all the bytes have been written. Returns the total number of
// bytes sent or -1 if an error occurs.
{
  bytes_moved = 0;           // Reset the byte counter
  int num_moved = 0;         // Actual number of bytes written
  int num_req = (int)bytes;  // Number of bytes requested 
  char *p = (char *)buf;     // Pointer to the buffer

  while(bytes_moved < bytes) { // Loop unitl the buffer is empty
    if((num_moved = send(s, p, num_req-bytes_moved, flags)) > 0) {
      bytes_moved += num_moved;  // Increment the byte counter
      p += num_moved;            // Move the buffer pointer for the next read
    }
    if(num_moved < 0) {
      socket_error = vbSOCKET_TRANSMIT_ERROR;
      return -1; // An error occurred during the read
    }
  }
  
  return bytes_moved;
}

int vbSocket::RemoteRecv(void *buf, int bytes, int flags)
// Receive a block of data from a remote socket in blocking mode.
// Returns the total number of bytes received or -1 if an error occurs.
{
  bytes_read = 0;           // Reset the byte counter
  int num_read = 0;         // Actual number of bytes read
  int num_req = (int)bytes; // Number of bytes requested 
  char *p = (char *)buf;    // Pointer to the buffer

  while(bytes_read < bytes) { // Loop unitl the buffer is full
    if((num_read = recv(remote_socket, p, num_req-bytes_read, flags)) > 0) {
      bytes_read += num_read;   // Increment the byte counter
      p += num_read;            // Move the buffer pointer for the next read
    }
    if(num_read < 0) {
      socket_error = vbSOCKET_RECEIVE_ERROR;
      return -1; // An error occurred during the read
    }
  }
  
  return bytes_read;
}

int vbSocket::RemoteSend(const void *buf, int bytes, int flags)
// Send a block of data to a remote socket and do not return
// unitl all the bytes have been written. Returns the total 
// number of bytes received or -1 if an error occurs.
{
  bytes_moved = 0;           // Reset the byte counter
  int num_moved = 0;         // Actual number of bytes written
  int num_req = (int)bytes;  // Number of bytes requested 
  char *p = (char *)buf;     // Pointer to the buffer

  while(bytes_moved < bytes) { // Loop unitl the buffer is empty
    if((num_moved = send(remote_socket, p, num_req-bytes_moved, flags)) > 0) {
      bytes_moved += num_moved;  // Increment the byte counter
      p += num_moved;            // Move the buffer pointer for the next read
    }
    if(num_moved < 0) {
      socket_error = vbSOCKET_TRANSMIT_ERROR;
      return -1; // An error occurred during the read
    }
  }
  
  return bytes_moved;
}

void vbSocket::ShutDown(int how)
// Used to close and un-initialize a full-duplex socket.
{
  bytes_moved = 0;
  bytes_read = 0;
  is_connected = 0;
  is_bound = 0;

  // NOTE 05/25/2000: The last error is no longer reset in following
  // a close call to allow the that appllication can close the socket
  // and catch the last exception reported if the socket has to be
  // closed after a Bind(), Listen(), or any other initialization
  // failure.
  // socket_error = vbSOCKET_NO_ERROR;
 
  // 0 = User is no longer interested in reading data
  // 1 = No more data will be sent
  // 2 = No data is to be sent or received
  if(vbsocket != -1) shutdown(vbsocket, how);
  if(remote_socket != -1) shutdown(remote_socket, how);

#if defined (__WIN32__)
  WSACleanup();
#endif
}

void vbSocket::ShutDownSocket(int how)
// Used to close a full-duplex server side socket.
{
  // 0 = User is no longer interested in reading data
  // 1 = No more data will be sent
  // 2 = No data is to be sent or received
  if(vbsocket != -1) shutdown(vbsocket, how);
  vbsocket = -1;
}

void vbSocket::ShutDownRemoteSocket(int how)
// Used to close a full-duplex client side socket.
{
  // 0 = User is no longer interested in reading data
  // 1 = No more data will be sent
  // 2 = No data is to be sent or received
  if(remote_socket != -1) shutdown(remote_socket, how);
  remote_socket = -1;
}

void vbSocket::Close()
// Close any and un-initialize any bound sockets.
{
  bytes_moved = 0;
  bytes_read = 0;
  is_connected = 0;
  is_bound = 0;

  // NOTE 05/25/2000: The last error is no longer reset in following
  // a close call to allow the that appllication can close the socket
  // and catch the last exception reported if the socket has to be
  // closed after a Bind(), Listen(), or any other initialization
  // failure.
  // socket_error = vbSOCKET_NO_ERROR;
#if defined (__WIN32__)
  if(vbsocket != -1) closesocket(vbsocket);
  if(remote_socket != -1) closesocket(remote_socket);
  WSACleanup();
#elif defined (__UNIX__)
  if(vbsocket != -1) close(vbsocket);
  if(remote_socket != -1) close(remote_socket);
#else
#error You must define a target platform:\n __DOS__ __WIN32__ or __UNIX__
#endif
}

void vbSocket::CloseSocket()
// Close the server side socket
{
#if defined (__WIN32__)
  if(vbsocket != -1) closesocket(vbsocket);
#elif defined (__UNIX__)
  if(vbsocket != -1) close(vbsocket);
#else
#error You must define a target platform:\n __DOS__ __WIN32__ or __UNIX__
#endif
  vbsocket = -1;
}

void vbSocket::CloseRemoteSocket()
// Close the client socket
{
#if defined (__WIN32__)
  if(remote_socket != -1) closesocket(remote_socket);
#elif defined (__UNIX__)
  if(remote_socket != -1) close(remote_socket);
#else
#error You must define a target platform:\n __DOS__ __WIN32__ or __UNIX__
#endif
  remote_socket = -1;
}

int vbSocket::Listen(int max_connections)
// Listen for connections if configured as a server.
// The "max_connections" variable determines how many
// pending connections the queue will hold. Returns -1
// if an error occurs.
{
  int rv = listen(vbsocket,         // Bound socket
		  max_connections); // Number of connection request queue
  if(rv < 0) socket_error = vbSOCKET_LISTEN_ERROR;
  return rv;
}

vbsSocketType vbSocket::Accept()
// Accept a connect from a remote socket. An Accept() 
// call blocks the server until the a client requests 
// service. Returns a valid socket descriptor or -1 
// if an error occurs.
{
  int addr_size = sizeof(remote_sin); // Length of client address
  remote_socket = accept(vbsocket, (struct sockaddr *)&remote_sin, &addr_size);
  if(remote_socket < 0) socket_error = vbSOCKET_ACCEPT_ERROR;
  return remote_socket;
}

int vbSocket::RecvFrom(void *buf, int bytes, int flags)
// Receive a block of data from a remote datagram socket 
// and do not return until all the bytes have been read. 
// Returns the total number of bytes receievd or -1 if 
// an error occurs.
{
  int addr_size = sizeof(remote_sin); // Length of client address
  bytes_read = 0;           // Reset the byte counter
  int num_read = 0;         // Actual number of bytes read
  int num_req = (int)bytes; // Number of bytes requested 
  char *p = (char *)buf;    // Pointer to the buffer

  while(bytes_read < bytes) { // Loop unitl the buffer is full
    if((num_read = recvfrom(vbsocket, p, num_req-bytes_read, flags, \
			    (struct sockaddr *)&remote_sin, &addr_size)) > 0) {
      bytes_read += num_read;   // Increment the byte counter
      p += num_read;            // Move the buffer pointer for the next read
    }
    if(num_read < 0) {
      socket_error = vbSOCKET_RECEIVE_ERROR;
      return -1; // An error occurred during the read
    }
  }
  return bytes_read;
}

int vbSocket::SendTo(void *buf, int bytes, int flags)
// Send a block of data to a datagram socket and do not return
// unitl all the bytes have been written. Returns the total number
// of bytes sent or -1 if an error occurs.
{
  int addr_size = sizeof(sin); // Length of address
  bytes_moved = 0;             // Reset the byte counter
  int num_moved = 0;           // Actual number of bytes written
  int num_req = (int)bytes;    // Number of bytes requested 
  char *p = (char *)buf;       // Pointer to the buffer

  while(bytes_moved < bytes) { // Loop unitl the buffer is full
    if((num_moved = sendto(vbsocket, p, num_req-bytes_moved, flags, \
			   (const struct sockaddr *)&sin, addr_size)) > 0) {
      bytes_moved += num_moved;  // Increment the byte counter
      p += num_moved;            // Move the buffer pointer for the next read
    }
    if(num_moved < 0) {
      socket_error = vbSOCKET_TRANSMIT_ERROR;
      return -1; // An error occurred during the read
    }
  }
  return bytes_moved;
}

int vbSocket::GetSockName()
// Retrieves the current name for the specified socket descriptor.
// It is used on a bound and/or connected socket and returns the
// local association. This function is especially useful when a
// connect call has been made without doing a bind first in which
// case this function provides the only means by which you can
// determine the local association which has been set by the system.
// Returns -1 if an error occurs.
{
  int namelen = sizeof(vbsSocketAddress);
  int rv = getsockname(vbsocket, (struct sockaddr *)&sin, &namelen);
  if(rv < 0) socket_error = vbSOCKET_HOSTNAME_ERROR;
  return rv;
}

int vbSocket::GetServByName(char *name, char *protocol)
// Get service information corresponding to a service name and protocol.
// Returns -1 if an unknown service or protocol is requested.
{
  // If the "protocol" pointer is NULL, getservbyname returns
  // the first service entry for which the name matches the s_name
  // or one of the s_aliases. Otherwise getservbyname matches both
  // the name and the proto.
  sp = getservbyname(name, protocol);
  if(sp == 0) {
    socket_error = vbSOCKET_PROTOCOL_ERROR;
    return -1;
  }
  sin.sin_port = sp->s_port;
  return 0;
}

int vbSocket::GetServByPort(int port, char *protocol)
// Get service information corresponding to a service name and protocol.
// Returns -1 if an unknown service or protocol is requested.
{
  // If the "protocol" pointer is NULL, getservbyport returns the
  // first service entry for which the port matches the s_port.
  // Otherwise getservbyport matches both the port and the proto.
  sp = getservbyport(port, protocol);
  if(sp == 0) {
    socket_error = vbSOCKET_PROTOCOL_ERROR;
    return -1;
  }
  sin.sin_port = sp->s_port;
  return 1;
}

int vbSocket::GetPortNumber()
// Return the port number actually set by the system. Use this fucntion
// after a call to vbSocket::GetSockName();
{
  return ntohs(sin.sin_port);
}

int vbSocket::GetRemotePortNumber()
// Return the port number of the client socket.
{
  return ntohs(remote_sin.sin_port);
}

vbsAddressFamily vbSocket::GetAddressFamily()
// Returns the address family of this socket
{
  return sin.sin_family;
}

vbsAddressFamily vbSocket::GetRemoteAddressFamily()
// Returns the address family of the remote socket.
{
  return remote_sin.sin_family;  
}

int vbSocket::GetHostName(char *hs)
// Pass back the host name of this machine in the "hs" variable.
// A memory buffer equal to "vbsMAX_NAME_LEN" must be pre-allocated
// prior to using this function. Return -1 is an error occurs.
{
  // Prevent crashes if memory has not been allocated
  if(!hs) hs = new char[vbsMAX_NAME_LEN]; 
  int rv = gethostname(hs, vbsMAX_NAME_LEN);
  if(rv < 0) socket_error = vbSOCKET_HOSTNAME_ERROR;
  return rv;
}

int vbSocket::GetRemoteHostName(char *hs)
// Pass back the host name the client in the "hs" variable.
// A memory buffer equal to "vbsMAX_NAME_LEN" must be pre-allocated
// prior to using this function. Return -1 is an error occurs.
{
  char *s = inet_ntoa(remote_sin.sin_addr);
  if(s == 0) {
    socket_error = vbSOCKET_HOSTNAME_ERROR;
    return -1;
  }

  // Prevent crashes if memory has not been allocated
  if(!hs) hs = new char[vbsMAX_NAME_LEN]; 

  strcpy(hs, s);
  return 1;
}

int vbSocket::RawRead(void *buf, int bytes, int flags)
// Receive a block of data from the bound socket. NOTE: A
// raw read may return before all the bytes have been read.
// Returns -1 if an error occurs.
{
  bytes_read = 0;
  bytes_read = recv(vbsocket, (char *)buf, bytes, flags);
  if(bytes_read < 0) socket_error = vbSOCKET_RECEIVE_ERROR;
  return bytes_read;
}

int vbSocket::RawWrite(const void *buf, int bytes, int flags)
// Send a block of data to the bound socket. NOTE: A raw write
// may return before all the bytes have been written. Returns -1
// if an error occurs.
{
  bytes_moved = 0;
  bytes_moved = send(vbsocket, (char *)buf, bytes, flags);
  if(bytes_moved < 0) socket_error = vbSOCKET_TRANSMIT_ERROR;
  return bytes_moved;
}

int vbSocket::RawRead(vbsSocketType s, void *buf, int bytes, int flags)
// Receive a block of data from a specifed socket. NOTE: A
// raw read may return before all the bytes have been read.
// Returns -1 if an error occurs.
{
  bytes_read = 0;
  bytes_read = recv(s, (char *)buf, bytes, flags);
  if(bytes_read < 0) socket_error = vbSOCKET_RECEIVE_ERROR;
  return bytes_read;
}

int vbSocket::RawWrite(vbsSocketType s, const void *buf, int bytes,
		      int flags)
// Send a block of data to a spcifed socket socket. NOTE: A raw write
// may return before all the bytes have been written. Returns -1 if an
// error occurs.
{
  bytes_moved = 0;
  bytes_moved = send(s, (char *)buf, bytes, flags);
  if(bytes_moved < 0) socket_error = vbSOCKET_TRANSMIT_ERROR;
  return bytes_moved;
}

int vbSocket::RawRemoteRead(void *buf, int bytes, int flags)
// Receive a block of data from a remote socket.  NOTE: A
// raw read may return before all the bytes have been read.
// Returns -1 if an error occurs.
{
  bytes_read = 0;
  bytes_read = recv(remote_socket, (char *)buf, bytes, flags);
  if(bytes_read < 0) socket_error = vbSOCKET_RECEIVE_ERROR;
  return bytes_read;
}

int vbSocket::RawRemoteWrite(const void *buf, int bytes, int flags)
// Send a block of data to a remote socket. NOTE: A raw write
// may return before all the bytes have been written. Returns
// -1 if an error occurs.
{
  bytes_moved = 0;
  bytes_moved = send(remote_socket, (char *)buf, bytes, flags);
  if(bytes_moved < 0) socket_error = vbSOCKET_TRANSMIT_ERROR;
  return bytes_moved;
}

int vbSocket::RawReadFrom(void *buf, int bytes, int flags)
// Receive a block of data from a remote datagram socket. NOTE: 
// A raw read may return before all the bytes have been read.
// Returns -1 if an error occurs.
{
  bytes_read = 0;
  int addr_size = sizeof(remote_sin); // Length of client address
  bytes_read = recvfrom(vbsocket, (char *)buf, bytes, flags,
			(struct sockaddr *)&remote_sin, &addr_size);
  if(bytes_read < 0) socket_error = vbSOCKET_RECEIVE_ERROR;
  return bytes_read;
}

int vbSocket::RawWriteTo(void *buf, int bytes, int flags)
// Send a block of data to a datagram socket. NOTE: A raw write
// may return before all the bytes have been written. Returns -1
// if an error occurs.
{
  bytes_moved = 0;
  int addr_size = sizeof(sin); // Length of address
  bytes_moved = sendto(vbsocket, (char *)buf, bytes, flags,
		       (const struct sockaddr *)&sin, addr_size);
  if(bytes_moved < 0) socket_error = vbSOCKET_TRANSMIT_ERROR;
  return bytes_moved;
}

const char *vbSocket::SocketExceptionMessage()
// Returns a null termainated string that can
// be use to log or print a socket execption.
{
  int error = (int)socket_error;

  // Find the corresponding message in the exception array
  return vbsSocketExceptionMessages[error];
}

// --------------------------------------------------------------
// MS Windows WinSock extensions
// --------------------------------------------------------------
#if defined (__WIN32__)
int vbSocket::InitWinSock(WORD ver)
// Initialize a specified winsock version.
// Returns true if an error occurs.
{
  wVersionRequested = ver;
  if(WSAStartup(wVersionRequested, &wsaData)) {
    socket_error = vbSOCKET_WINSOCKINIT_ERROR;
    return 1;
  }
  return 0;
}

int vbSocket::InitWinSock()
// Initialize winsock with the vbSocket default version.
// Returns true if an error occurs.
{
  wVersionRequested = MAKEWORD(1,1);
  if(WSAStartup(wVersionRequested, &wsaData)) {
    socket_error = vbSOCKET_WINSOCKINIT_ERROR;
    return 1;
  }
  return 0;
}

int vbSocket::CheckWinSockVersion()
// Call after initializing the winsock.dll. Returns true if the
// WinSock version is supported.
{
  if(wsaData.wVersion != wVersionRequested) {
    socket_error = vbSOCKET_WINSOCKVERSION_ERROR;
    return 0;
  }
  return 1;
}

void vbSocket::ReleaseWinSock()
// Release the winsock.dll.
{
  WSACleanup();
}
#endif // MS Windows WinSock extensions
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

