// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: testprog.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/25/2000
// Date Last Modified: 05/09/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

vbMutex example program demonstrating how to protect access to
shared data.
*/
// ----------------------------------------------------------- //   
#include <iostream.h>
#include "vbthread.h"
#include "vbmutex.h"

// Constants
const int NUMTHREADS = 3;

// Mutex that will be shared by threads within this process
vbMutex mutex;

// Shared data
int sharedData1 = 0;
int sharedData2 = 0;

class SimpleThread : public vbThread
{
public:
  SimpleThread() { }
  ~SimpleThread() { }
  
public:
  void Begin();
  void End();
  
private: // Base class interface
  void *ThreadEntryRoutine(vbThread_t *thread);

public:
  vbThread_t *thread[NUMTHREADS];
};

void *SimpleThread::ThreadEntryRoutine(vbThread_t *thread)
{
  cout << "Entering thread" << endl;
  mutex.MutexLock();
  
  //********** Critical Section *******************//
  cout << "Start critical section, holding lock" << endl;

  // Access to shared data goes here 
  ++sharedData1; --sharedData2;
  cout << "sharedData1 = " << sharedData1 << ", sharedData2 = " << sharedData2
       << endl;
  
  cout << "End critical section, release lock" << endl;
  //********** Critical Section *******************//

  mutex.MutexUnlock();
  return 0;
}

void SimpleThread::Begin()
{
  for(int i = 0; i < NUMTHREADS; ++i) thread[i] = CreateThread();
}

void SimpleThread::End()
{
  for(int i = 0; i < NUMTHREADS; ++i) JoinThread(thread[i]);
}

int main()
{
  SimpleThread t;
  
  cout << "Hold Mutex to prevent access to shared data" << endl;
  cout << "sharedData1 = " << sharedData1 << ", sharedData2 = " << sharedData2
       << endl;
  
  mutex.MutexLock();
 
  cout << "Create/start threads" << endl;
  t.Begin();
  
  cout << "Wait a until we are done with the shared data" << endl;
  t.sSleep(3);

  cout << "Unlock shared data" << endl;
  mutex.MutexUnlock();

  cout << "Wait for the threads to complete, and release their resources"
       << endl;
  t.End();
  
  return 0; // Exit the process
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

 
