// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: testprog.cpp
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/25/2000
// Date Last Modified: 05/09/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Test program demonstrating the basic operation of the vbThread
class using semaphores.
*/
// ----------------------------------------------------------- //   
#include <iostream.h>
#include <stdlib.h>
#include "vbthread.h"
#include "vbsema.h"

// Constants
const int NUM_THREADS = 3;

// Global semaphore object
vbSemaphore sema;

// Shared data 
int sharedData1 = 0;
int sharedData2 = 0;
 
// Class derived from the abstract vbThread base class 
class SimpleThread : public vbThread
{
public:
  SimpleThread() { }
  ~SimpleThread() { }
  
private: // Base class interface
  void *ThreadEntryRoutine(vbThread_t *thread);
};

void *SimpleThread::ThreadEntryRoutine(vbThread_t *thread)
{
  cout << "Entering thread" << endl;
  if(sema.SemaphoreWait() != 0) {
    cout << sema.SemaphoreExceptionMessage() << endl;
    return ExitThread(thread, 1);
  }

  // ********** Critical Section ******************* //
  cout << "Start critical section, holding semaphore" << endl;

  // Access to shared data goes here
  ++sharedData1; --sharedData2;
  cout << "sharedData1 = " << sharedData1 << ", sharedData2 = " << sharedData2
       << endl;

  cout << "End critical section, release semaphore" << endl;
  // ********** Critical Section ******************* //

  if(sema.SemaphorePost() != 0) {
    cout << sema.SemaphoreExceptionMessage() << endl;
    return ExitThread(thread, 1);
  }

  return 0;
}
 
int main(int argc, char **argv)
{
  SimpleThread t;
  vbThread_t *thread[NUM_THREADS];
  int i;
 
  cout << "Wait on semaphore to prevent access to shared data" << endl;
  cout << "sharedData1 = " << sharedData1 << ", sharedData2 = " << sharedData2
       << endl;

  if(sema.SemaphoreWait() != 0) {
    cout << sema.SemaphoreExceptionMessage() << endl;
    return 0;
  }

  cout << "Creating " << NUM_THREADS << " threads" << endl;
  for(i = 0; i < NUM_THREADS; ++i) thread[i] = t.CreateThread();
 
  cout << "Wait a bit until we are done with the shared data" << endl;
  t.sSleep(3);

  cout << "Unlock shared data" << endl;
  if(sema.SemaphorePost() != 0) {
    cout << sema.SemaphoreExceptionMessage() << endl;
    return 0;
  }

  cout << "Wait for the threads to complete, and release their resources" << endl;
  for(i = 0; i < NUM_THREADS; ++i) t.JoinThread(thread[i]);

  return 0;
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
