// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: thrpool.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/31/2000
// Date Last Modified: 05/30/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The thrPool class is used to pool threads of a single or multiple
processes together. Each thread pool consists of a doubly linked
list of thrPoolNode nodes. A thrPoolNode node holds a pointer to
a vbThread_t object plus a pointer to previous and next node in
the list.
*/
// ----------------------------------------------------------- //   
#ifndef __VBTHREAD_POOL_HPP__
#define __VBTHREAD_POOL_HPP__

// Foward class calling conventions
class vbThread_t;
class thrPool;

// Thread Pool Node Class
class thrPoolNode
{
private:
  friend class thrPool;

public:
  thrPoolNode() { next = prev = 0; thread = 0; }
  thrPoolNode(vbThread_t *t) { next = prev = 0; thread = t; }
  ~thrPoolNode() { }
  
private: // Disallow copying and assignmnet
  thrPoolNode(const thrPoolNode &ob) { }
  void operator=(const thrPoolNode &ob) { }
  
public:
  thrPoolNode *GetNext() { return next; }  
  thrPoolNode *GetPrev() { return prev; } 
  vbThread_t *GetThreadPtr() { return thread; } 
  vbThread_t *GetThreadPtr() const { return thread; } 
  void SetThreadPtr(vbThread_t *t) { thread = t; }
  void SetThreadPtr(const vbThread_t *t) { thread = (vbThread_t *)t; }
  void SetNext(thrPoolNode *n) { next = n; }  
  void SetNext(const thrPoolNode *n) { next = (thrPoolNode *)n; }  
  void SetPrev(thrPoolNode *p) { prev = p; }
  void SetPrev(const thrPoolNode *p) { prev = (thrPoolNode *)p; }
  
private:
  vbThread_t *thread; // Pointer to the this node's thread
  thrPoolNode *next;  // Pointer to the next node in the list
  thrPoolNode *prev;  // Pointer to the previous node in the list
};

// Thread Pool Class
class thrPool
{
public:
  thrPool() { head = tail = 0; }
  ~thrPool();

private: // Disallow copying and assignment
  thrPool(const thrPool &ob) { }
  void operator=(const thrPool &ob) { }

public: // Pointer manipulation operations
  void DetachNode(thrPoolNode *);
  void MoveToFront(thrPoolNode *n);
  void MoveToBack(thrPoolNode *n);
  void MoveBefore(thrPoolNode *pos, thrPoolNode *n);
  void MoveAfter(thrPoolNode *pos, thrPoolNode *n);
  void InsertBefore(thrPoolNode *pos, thrPoolNode *n);
  void InsertAfter(thrPoolNode *pos, thrPoolNode *n);
  void InsertAtHead(thrPoolNode *n);
  void InsertAtTail(thrPoolNode *n);

public: // Add and remove functions
  thrPoolNode *AddThread(vbThread_t *thread);
  thrPoolNode *AddToFront(vbThread_t *t);
  thrPoolNode *AddToBack(vbThread_t *t);
  thrPoolNode *AddAfter(thrPoolNode *pos, vbThread_t *t);
  thrPoolNode *AddBefore(thrPoolNode *pos, vbThread_t *t);
  void ClearPool(); // Empty the pool without checking the thread status
  void MakeEmpty(); // Called only if all the nodes have been removed 
  vbThread_t *RemoveThread(vbThread_t *t);
  vbThread_t *RemoveNode(thrPoolNode *n);
  vbThread_t *RemoveHead();
  vbThread_t *RemoveTail();

public: // Search functions
  thrPoolNode *FindThread(vbThread_t *t, thrPoolNode *ptr = 0);

private: // Allocation/deallocation functions
  vbThread_t *FreeNode(thrPoolNode *n);
  thrPoolNode *AllocNode(vbThread_t *t);
  
public: // Node position functions
  int IsEmpty() { return head == 0; }
  int IsEmpty() const { return (const thrPoolNode *)head == 0; }
  thrPoolNode *GetHead() { return head; }
  thrPoolNode *GetTail() { return tail; }
  thrPoolNode *GetHead() const { return head; }
  thrPoolNode *GetTail() const { return tail; }
  void SetHead(thrPoolNode *h) { head = h; }
  void SetHead(const thrPoolNode *h) { head = (thrPoolNode *)h; }
  void SetTail(thrPoolNode *t) { tail = t; }
  void SetTail(const thrPoolNode *t) { tail = (thrPoolNode *)t; }
  
private:
  thrPoolNode *head; // First node in the list
  thrPoolNode *tail; // Last node in the list
};

#endif // __VBTHREAD_POOL_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
