// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: vbmutex.h
// Compiler Used: MSVC40, egcs-2.90.29, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 03/25/2000
// Date Last Modified: 05/30/2000
// Copyright (c) 2000 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD and VBS C++ classes are copyright (c) 1997 and 2000 by 
Douglas M. Gaer. All those who put this code or its derivatives 
in a commercial product MUST mention this copyright in their
documentation for users of the products in which this code or its 
derivative classes are used. Otherwise, you have the freedom to 
redistribute verbatim copies of this source code, adapt it to your 
specific needs, or improve the code and release your improvements 
to the public provided that the modified files carry prominent 
notices stating that you changed the files and the date of any
change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

Mutual exclusion class used to provide simple lock primitives
that can be used to control access to a shared resource. A mutex
is used to cause other threads to wait while the thread holding
the mutex executes code in a critical section.
*/
// ----------------------------------------------------------- //   
#ifndef __VBMUTEX_HPP__
#define __VBMUTEX_HPP__

#include "thrtypes.h"
#include "thelpers.h"

class vbMutex
{
public:
  vbMutex(vbProcessType type = vbPROCESS_PRIVATE); 
  ~vbMutex();

private: // Disallow copying and assignment
  vbMutex(const vbMutex &ob) { } 
  void operator=(const vbMutex &ob) { }

public:
  int MutexInit(vbProcessType type = vbPROCESS_PRIVATE);
  int MutexDestroy();
  int MutexLock();
  int MutexUnlock();
  int MutexTryLock();
  int IsLocked() { return mutex_lock > 0; }
  int NumLocks() { return mutex_lock; }
  vbMutex_t *GetMutex() { return &mutex; }
  vbMutexError GetMutexError() { return mutex.mutex_error; }
  vbProcessType GetMutexProcessType() { return mutex.process_type; }
  
public: // Mutex status/debug message functions
  const char *MutexExceptionMessage();

private:
  int mutex_lock;
  vbMutex_t mutex;
};

#endif // __VBMUTEX_HPP__
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
  

